from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.profile_enum import ProfileEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.object_links import ObjectLinks
    from ..models.patch_profile_response_data_attributes import PatchProfileResponseDataAttributes
    from ..models.patch_profile_response_data_relationships import PatchProfileResponseDataRelationships


T = TypeVar("T", bound="PatchProfileResponseData")


@_attrs_define
class PatchProfileResponseData:
    """
    Attributes:
        type (ProfileEnum):
        attributes (PatchProfileResponseDataAttributes):
        links (ObjectLinks):
        id (Union[Unset, str]): Primary key that uniquely identifies this profile. Generated by Klaviyo. Example:
            01GDDKASAP8TKDDA2GRZDSVP4H.
        relationships (Union[Unset, PatchProfileResponseDataRelationships]):
    """

    type: ProfileEnum
    attributes: "PatchProfileResponseDataAttributes"
    links: "ObjectLinks"
    id: Union[Unset, str] = UNSET
    relationships: Union[Unset, "PatchProfileResponseDataRelationships"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        attributes = self.attributes.to_dict()

        links = self.links.to_dict()

        id = self.id

        relationships: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.relationships, Unset):
            relationships = self.relationships.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "attributes": attributes,
                "links": links,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if relationships is not UNSET:
            field_dict["relationships"] = relationships

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.object_links import ObjectLinks
        from ..models.patch_profile_response_data_attributes import PatchProfileResponseDataAttributes
        from ..models.patch_profile_response_data_relationships import PatchProfileResponseDataRelationships

        d = src_dict.copy()
        type = ProfileEnum(d.pop("type"))

        attributes = PatchProfileResponseDataAttributes.from_dict(d.pop("attributes"))

        links = ObjectLinks.from_dict(d.pop("links"))

        id = d.pop("id", UNSET)

        _relationships = d.pop("relationships", UNSET)
        relationships: Union[Unset, PatchProfileResponseDataRelationships]
        if isinstance(_relationships, Unset):
            relationships = UNSET
        else:
            relationships = PatchProfileResponseDataRelationships.from_dict(_relationships)

        patch_profile_response_data = cls(
            type=type,
            attributes=attributes,
            links=links,
            id=id,
            relationships=relationships,
        )

        patch_profile_response_data.additional_properties = d
        return patch_profile_response_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
