from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.profile_enum import ProfileEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.profile_meta import ProfileMeta
    from ..models.profile_partial_update_query_resource_object_attributes import (
        ProfilePartialUpdateQueryResourceObjectAttributes,
    )


T = TypeVar("T", bound="ProfilePartialUpdateQueryResourceObject")


@_attrs_define
class ProfilePartialUpdateQueryResourceObject:
    """
    Attributes:
        type (ProfileEnum):
        id (str): Primary key that uniquely identifies this profile. Generated by Klaviyo. Example:
            01GDDKASAP8TKDDA2GRZDSVP4H.
        attributes (ProfilePartialUpdateQueryResourceObjectAttributes):
        meta (Union[Unset, ProfileMeta]):
    """

    type: ProfileEnum
    id: str
    attributes: "ProfilePartialUpdateQueryResourceObjectAttributes"
    meta: Union[Unset, "ProfileMeta"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        id = self.id

        attributes = self.attributes.to_dict()

        meta: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.meta, Unset):
            meta = self.meta.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "id": id,
                "attributes": attributes,
            }
        )
        if meta is not UNSET:
            field_dict["meta"] = meta

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.profile_meta import ProfileMeta
        from ..models.profile_partial_update_query_resource_object_attributes import (
            ProfilePartialUpdateQueryResourceObjectAttributes,
        )

        d = src_dict.copy()
        type = ProfileEnum(d.pop("type"))

        id = d.pop("id")

        attributes = ProfilePartialUpdateQueryResourceObjectAttributes.from_dict(d.pop("attributes"))

        _meta = d.pop("meta", UNSET)
        meta: Union[Unset, ProfileMeta]
        if isinstance(_meta, Unset):
            meta = UNSET
        else:
            meta = ProfileMeta.from_dict(_meta)

        profile_partial_update_query_resource_object = cls(
            type=type,
            id=id,
            attributes=attributes,
            meta=meta,
        )

        profile_partial_update_query_resource_object.additional_properties = d
        return profile_partial_update_query_resource_object

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
